unit classBaseStateMachine;

interface

uses
  SysUtils, Generics.collections;

type
  TStateMethod = function:integer of object;

  TBaseStatemachine = class(TObject)
  protected
    fCurrentState     : integer;
    fStateList        : TDictionary;

    procedure addState(stateIndex:integer;aStateMethod:TStateMethod);
  public
    constructor create;
    destructor Destroy; override;

    procedure process;

    procedure initialiseStates; virtual; abstract;

    property currentState:integer read fCurrentState write fCurrentState;
  end;

implementation

procedure TBaseStateMachine.addState(stateIndex:integer;aStateMethod:TStateMethod);
begin
  if (fStateList.ContainsKey(stateIndex)) then
  begin
    raise exception.createFmt('%s.addState - Add failed, duplicate state %d',[self.className,stateIndex]);
  end;

  fStateList.Add(stateIndex,aStateMethod);
end;

constructor TBaseStateMachine.create;
begin
  inherited;

  fStateList:=TDictionary.create;
  fCurrentState:=0;

  initialiseStates;
end;

destructor TBaseStateMachine.Destroy;
begin
  try
    fStateList.free;
  except
  end;

  inherited;
end;

procedure TBaseStateMachine.process;
var
  nextState       : integer;
begin
  if (fStateList.ContainsKey(fCurrentState)) then
  begin
    nextState:=fStateList.Items[fCurrentState];

    if (nextState<>0) then
    begin
      fCurrentState:=nextState;
    end;
  end
  else
  begin
    raise exception.createFmt('%s.process failed - Unknown state %d',[self.className,fCurrentState]);
  end;
end;

end.